close all;

// ENDOGENOUS VARIABLES

var ii delta r b e piC Y d  C pii Q mc Deltta pC pStar varphiM tauM 
m X TB w z_piStar z_piStarX z_G z_epsilon z_iStar z_phiStar GammaP PsiP 
real_ii tax qf CA N dii ddelta gammaD gammaM gammaR varphiD tauD 
taxDCol taxD y_obs ;

// EXOGENOUS VARIABLES

varexo eps_epsilon eps_G eps_iStar eps_phiStar eps_piStar eps_piStarX ; 


// PARAMETERS

parameters 

// Monetary and XR simple policy rules
h_0 h_1 h_2 h_3               //Interest rate Simple Rule 
k_0 k_1 k_2 k_3 k_4           //Exchange rate Simple Rule 
j_0 j_1 j_2 j_3 j_4           //taxD Simple Rule 


// Persistence parameters in shock variables          
rhoPiStar rhoPiStarX rhoG rhoEpsilon rhoiStar rhoPhiStar rhoPiStarNX

// Great ratios  
gammaR_ss gammaD_ss gammaM_ss pCCtoy mtoy btoy GtopCC tbetoy 

// General                                                                           
piT betta sigmaC a_D thetta sigmaN alfa  bA bX kappaX 
varepsilon_L ELASTtauBarD_1 ELASTtauBarD_2
alpha_1 alpha_2 alpha_3 
xi betta_1 betta_2 betta_3 thettaC
EC_speed_N EC_speed_X 

// Steady state values of endogenous variables
Y_ss piStar_ss iStar_ss phiStar_ss pStar_ss  e_ss C_ss d_ss r_ss
m_ss b_ss mc_ss Q_ss GammaP_ss PsiP_ss N_ss pC_ss 
X_ss TB_ss Deltta_ss w_ss varphiM_ss tauM_ss tax_ss qf_ss taxD_ss
tauD_ss varphiD_ss taxDCol_ss ;

// PARAMETER VALUES

// Monetary and XR simple policy rules (these don't affect the SS)

h_0=0.5; h_1=0.8; h_2=0.5;   h_3=0.5;               
k_0=0.8; k_1=0.5; k_2=0.5;  k_3=0.5;  k_4=0.8 ;   
j_0=0.8; j_1=0.2; j_2=0.2;   j_3=0.8;  j_4=0.8 ;

// Calibrated steady state values of endogenous variables
%Y_ss=10.873;  
piStar_ss=1; iStar_ss=1.03^0.25; phiStar_ss=1.01^0.25; pStar_ss=1;

// Persistence parameters in shock variables (these don't affect the SS)

rhoEpsilon=0.8; rhoG=0.85; rhoiStar=0.7; rhoPhiStar=0.3;
rhoPiStar=0.912; rhoPiStarX= -0.072 ; rhoPiStarNX=0.266;

betta=0.99;            //Intertemporal discount factor                    

taxD_ss=0.001;

piT=1.03^0.25;             //Inflation target        
alfa=0.75;             //Probability of non-optimizing price
sigmaC=2;            //Constant relative risk aversion for consumption
sigmaN=1 ;           //Constant relative risk aversion for labor 
a_D=0.5;              //Home bias in consumption
thetta=6;              //Elasticity of substitution for domestic goods
thettaC=1.5;           //Elasticity of substitution between domestic and imported bundles
bA=0.5;                //Exogenous parameter for exports demand
varepsilon_L=1.02;     //Elasticity of money/consumption function w.r. to ii
gammaD_ss=0.408;         //Household Foreign Debt to GDP ratio 

ELASTtauBarD_1=10;     //Debt/GDP ratio elasticity of tauBarD 
ELASTtauBarD_2=0;      //Reserves/GDP ratio elasticity of tauBarD (in UIP)

EC_speed_N=0.556;      //Imports error correction speed 
EC_speed_X=-0.061;     //Exports error correction speed 
bX=1/(1-bA);           //Composite parameter for exports demand
kappaX=bA^(bA/(1-bA)); //Composite parameter for exports demand

// Risk premium parameters 
gammaR_ss=1.448;         //CB reserves to GDP ratio
alpha_1=((1-taxD_ss)*(1/(betta*phiStar_ss*iStar_ss/piStar_ss))-1)/((1+ELASTtauBarD_1)*(1+ELASTtauBarD_1+ELASTtauBarD_2));
alpha_2=(ELASTtauBarD_1/(1+ELASTtauBarD_1+ELASTtauBarD_2))/gammaD_ss;
alpha_3=((-ELASTtauBarD_2)/(1+ELASTtauBarD_1+ELASTtauBarD_2))/gammaR_ss;
tauD_ss=1+alpha_1/(1-alpha_2*gammaD_ss+alpha_3*gammaR_ss);                            
varphiD_ss=1+(tauD_ss-1)*(1+alpha_2*gammaD_ss/(1-alpha_2*gammaD_ss+alpha_3*gammaR_ss));

// Great ratios
N_ss=0.567;
Deltta_ss=((1-alfa)/((1-alfa*piT^(thetta))))*((1-alfa*piT^(thetta-1))/(1-alfa))^(thetta/(thetta-1));
Q_ss=N_ss/Deltta_ss;
tbetoy=((iStar_ss/piStar_ss)*phiStar_ss*tauD_ss-1)*gammaD_ss-(iStar_ss/piStar_ss-1)*gammaR_ss;
f_e = @(e_ss) ((kappaX*(e_ss*pStar_ss)^bX)*(a_D+(1-a_D)*(e_ss)^(1-thettaC))-(1-a_D)*e_ss^(1-thettaC))/a_D-tbetoy;
e_ss=fzero(f_e,0.9966);                       
Y_ss=Q_ss/(1-(1-bA)*kappaX*(e_ss*pStar_ss)^bX);
TB_ss=tbetoy*Y_ss/e_ss;
X_ss=Y_ss*kappaX*(e_ss*pStar_ss)^bX;



// Transactions cost parameters 

gammaM_ss=2.939;     //money/consumption ratio

GtopCC=0.362;            //Gov expenditure to private consumption ratio
betta_3=160;            //Should be >1/(varepsilon_L*(piT/betta-1))-1 
betta_2=(1/gammaM_ss)*(1/(((betta_3+1)*varepsilon_L*(piT/betta-1))-1));                        
betta_1=(1-betta/piT)*((1+betta_2*gammaM_ss)^(betta_3+1))/(betta_2*betta_3);          
tauM_ss=1+betta_1/(1+betta_2*gammaM_ss)^betta_3;
varphiM_ss=1+(tauM_ss-1)*(1+betta_3*((betta_2*gammaM_ss)/(1+betta_2*gammaM_ss))); 

pC_ss=(a_D+(1-a_D)*(e_ss)^(1-thettaC))^(1/(1-thettaC));    
pCCtoy=(1-kappaX*(e_ss*pStar_ss)^bX)*(pC_ss^(1-thettaC)/(a_D*GtopCC*tauM_ss));
C_ss=pCCtoy*Y_ss/pC_ss;         
mtoy=gammaM_ss*pCCtoy;     //money/GDP ratio
btoy=gammaR_ss-mtoy;       //bonds/GDP ratio

// Steady state values of endogenous variables
  
GammaP_ss=((Q_ss/(pC_ss*C_ss^sigmaC))/(1-betta*alfa*piT^(thetta-1)));
PsiP_ss=GammaP_ss/(((1-alfa*piT^(thetta-1))/(1-alfa))^(1/(thetta-1)));
mc_ss=PsiP_ss*(1-betta*alfa*((piT)^thetta))/((thetta/(thetta-1))*(Q_ss/(pC_ss*C_ss^sigmaC)));    
w_ss=mc_ss;
r_ss=gammaR_ss*Y_ss/e_ss;
b_ss=btoy*Y_ss;
qf_ss=((iStar_ss)-1/(piT/piStar_ss))*(e_ss*r_ss/piStar_ss)-(piT/betta-1)*(b_ss/piT);
d_ss=gammaD_ss*Y_ss/e_ss;
m_ss=mtoy*Y_ss;
xi=w_ss/(N_ss^sigmaN*pC_ss*C_ss^sigmaC*(varphiM_ss));
taxDCol_ss=taxD_ss*e_ss*d_ss;
tax_ss=(GtopCC-1)*tauM_ss*pC_ss*C_ss-qf_ss-taxDCol_ss;


// MODEL EQUATIONS

model;


// Simple policy rules 

// 1 Taylor rule

ii=(piT/betta)^(1-h_0)*(ii(-1))^h_0*(piC/piT)^h_1*(Y/Y_ss)^h_2*(e/e_ss)^h_3;

//For Pegged Exchange Rate (PER) policy use instead:
//b=b_ss;

// 2 Exchange rate depreciation rule:

delta=(piT/piStar_ss)^(1-k_0)*delta(-1)^k_0*(piC/piT)^k_1*(Y/Y_ss)^k_2*(e/e_ss)^k_3*(gammaR/gammaR_ss)^k_4;

//For Floating Exchange Rate (FER) policy use instead:
//r=r_ss;


// 3 Tax on capital flows rule:

taxD=taxD_ss^(1-j_0)*taxD(-1)^j_0*(piC/piT)^j_1*(Y/Y_ss)^j_2*(e/e_ss)^j_3*(z_phiStar/log(phiStar_ss))^j_4;

//taxD=taxD_ss; // For Fixed TobinTax.


// 4 Central Bank balance sheet

b=e*r-m;

// 5 Money market clearing

m=(1/betta_2)*(((betta_1*betta_2*betta_3/(1-1/ii))^(1/(betta_3+1)))-1)*pC*C;    

// 6 Trade Balance (in foreign currency)

TB=(1/(a_D*e))*(pC^(1-thettaC)*X-(1-a_D)*e^(1-thettaC)*Y);

// 7 8 Current account (in foreign currency)

tauD=1+alpha_1/(1-alpha_2*gammaD+alpha_3*gammaR);

CA=(exp(z_iStar(-1))/exp(z_piStar)-1)*r(-1)-((exp(z_iStar(-1))/exp(z_piStar))*exp(z_phiStar(-1))*(tauD(-1))-1)*d(-1)+TB;

// 9 Balance of Payments  (in foreign currency)

r=d+CA+r(-1)-d(-1);

// 10 Exports (in foreign currency)

X=Y*kappaX*((e*pStar)^bX);

// 11 RER identity

e=e(-1)*(delta*exp(z_piStar)/pii);

// 12 13 Risk-adjusted uncovered interest parity 

varphiD=1+(tauD-1)*(1+alpha_2*gammaD/(1-alpha_2*gammaD+alpha_3*gammaR));

ii=exp(z_iStar)*exp(z_phiStar)*(varphiD/(1-taxD))*delta(+1); 

// 14 Labor market clearing

w=xi*N^sigmaN*pC*C^sigmaC*varphiM;

// 15 Hours worked

N=Q*Deltta/exp(z_epsilon);

// 16 Domestic goods market clearing

Q=Y-(1-bA)*X;

// 17 GDP DEFINITION

Y=a_D*exp(z_G)*tauM*pC^thettaC*C+X;         

// 18 Dynamics of Price dispersion

Deltta=alfa*pii^thetta*Deltta(-1)+(1-alfa)*((1-alfa*pii^(thetta-1))/(1-alfa))^(thetta/(thetta-1));

// 19-21 Domestic inflation Phillips equation 

GammaP=PsiP*((1-alfa*pii^(thetta-1))/(1-alfa))^(1/(thetta-1));

GammaP=Q/(pC*C^sigmaC)+betta*alfa*(pii(+1))^(thetta-1)*GammaP(+1);       

PsiP=(thetta/(thetta-1))*(Q/(pC*C^sigmaC))*mc+betta*alfa*((pii(+1))^thetta)*PsiP(+1);    

// 22 Real Marginal Cost

mc=w/exp(z_epsilon);

// 23 Transactions cost (net)

tauM=1+betta_1/(1+betta_2*gammaM)^betta_3;

// 24 Transactions cost effect on the Euler equation (gross)

varphiM=1+(tauM-1)*(1+betta_3*((betta_2*gammaM)/(1+betta_2*gammaM))); 

// 25 Consumption relative price

pC^(1-thettaC)=a_D+(1-a_D)*e^(1-thettaC);   

// 26 Consumption inflation identity

piC=pii*pC/pC(-1);    

// 27 Euler equation

C^(-sigmaC)/varphiM=betta*ii*(C(+1)^(-sigmaC)/piC(+1))/varphiM(+1);

// 28 External Terms of Trade

pStar=pStar(-1)*exp(z_piStarX)/exp(z_piStar);     


// VARIABLES SUBJECT TO STOCHASTIC SHOCKS BLOCK (Shocked variables are in logs)

// 29 30 Cointegrated RW prices

z_piStarX=z_piStarX(-1)*rhoPiStarX+log(pStar(-1))*EC_speed_X+z_piStar(-1)*rhoPiStarNX+eps_piStarX;

z_piStar=z_piStar(-1)*rhoPiStar+log(pStar(-1))*EC_speed_N+eps_piStar;

// 31 Government expenditure shock 

z_G=(z_G(-1))*rhoG+log(GtopCC)*(1-rhoG)+eps_G;

// 32 Transitory productivity shock 

z_epsilon=z_epsilon(-1)*rhoEpsilon+eps_epsilon;

// 33 RW riskfree interest rate shock 

z_iStar=(z_iStar(-1))*rhoiStar+log(iStar_ss)*(1-rhoiStar)-eps_iStar; 

// 34 RW Financing liquidity shock 

z_phiStar=z_phiStar(-1)*rhoPhiStar+log(phiStar_ss)*(1-rhoPhiStar)-eps_phiStar; 

// 35 Real interest rate

real_ii=ii/piC(+1);

// 36 Quasi-fiscal surplus

qf=(exp(z_iStar(-1))-1/delta)*e*r(-1)/exp(z_piStar)-(ii(-1)-1)*b(-1)/pii;

// 37 38 Tax collection

taxDCol=taxD*e*d;

tax=(exp(z_G)-1)*tauM*pC*C-qf-taxDCol;

// 39 40 Policy instruments differences for ramsey

dii=ii-ii(-1);

ddelta=delta-delta(-1);

// 41-43 Great ratios

gammaD=d*e/Y;

gammaM=m/(pC*C);

gammaR=e*r/Y;     y_obs=Y/Y(-1);

end;

initval;

ii=piT/betta; delta=piT/piStar_ss;  taxD=taxD_ss; 
r=r_ss; b=b_ss; e=e_ss; piC=piT; Y=Y_ss; 
d=d_ss;  C=C_ss; pii=piT; Q=Q_ss;  mc=mc_ss;  Deltta=Deltta_ss; pC=pC_ss; 
pStar=pStar_ss; varphiM=varphiM_ss; tauM=tauM_ss;  m=m_ss; X=X_ss; TB=TB_ss; w=w_ss; 
z_piStar=log(piStar_ss);  z_piStarX=log(piStar_ss);  z_G=log(GtopCC); z_epsilon=log(1); 
z_iStar=log(iStar_ss); z_phiStar=log(phiStar_ss); GammaP=GammaP_ss; PsiP=PsiP_ss;
real_ii=ii/piC; tax=tax_ss; qf=qf_ss; CA=0; N=N_ss; dii=0; ddelta=0; 
gammaD=gammaD_ss; gammaM=gammaM_ss; gammaR=gammaR_ss;  
varphiD=varphiD_ss; tauD=tauD_ss; taxDCol=taxDCol_ss; y_obs=1;

end;

steady;  
check;model_diagnostics;

shocks;

var eps_epsilon;        stderr  0.01 ;   
var eps_G;              stderr  0.006 ;  
var eps_iStar;          stderr  0.018;    
var eps_phiStar;        stderr  0.05 ; 
var eps_piStar;         stderr  0.062 ;   
var eps_piStarX;        stderr  0.026 ;   

end;



estimated_params;
h_0, beta_pdf,   0.5, 0.1;
h_1, beta_pdf,   0.8, 0.1;
h_2, beta_pdf,   0.2, 0.1;
h_3, beta_pdf,   0.5, 0.1;
k_0, beta_pdf,   0.8, 0.1;
k_1, beta_pdf,   0.5, 0.1;
k_2, beta_pdf,   0.5, 0.1;
k_3, beta_pdf,   0.5, 0.1;
k_4, beta_pdf,   0.8, 0.1;
j_0, beta_pdf,   0.5, 0.1;
j_1, beta_pdf,   0.2, 0.1;
j_2, beta_pdf,   0.2, 0.1;
j_3, beta_pdf,   0.8, 0.1;
j_4, beta_pdf,   0.8, 0.1;

alfa,0.75, beta_pdf, 0.66, 0.1;
thettaC, gamma_pdf, 1.5,0.1;
%betta_3, gamma_pdf, 160,10;

rhoEpsilon,beta_pdf, 0.8, 0.1;
rhoG, beta_pdf, 0.85, 0.1;
rhoiStar, beta_pdf, 0.7, 0.1;
rhoPhiStar, beta_pdf, 0.3, 0.1;


stderr eps_epsilon, inv_gamma_pdf,0.01,inf ;   
stderr eps_G,inv_gamma_pdf,0.01,inf;  
stderr eps_iStar,inv_gamma_pdf,0.01,inf;    
stderr eps_phiStar,inv_gamma_pdf,0.01,inf; 
stderr eps_piStar,inv_gamma_pdf,0.01,inf;   
stderr eps_piStarX,inv_gamma_pdf,0.01,inf;


end;

estimated_params_init(use_calibration);
end;


varobs delta y_obs pii z_piStar z_iStar;
estimation(mode_check, datafile=Obs_renewed, xls_sheet=Final ,mode_compute=4,mh_replic=2000);


stoch_simul(order=1,irf=40,solve_algo=2,irf_shocks=(eps_phiStar))  ;